@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2009 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2018 HP Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,8 +26,8 @@ import getopt
 import os
 
 # Local
-from g import *
-import tui, utils, device
+from .g import *
+from . import tui, utils, device
 
 USAGE_FLAG_NONE = 0x00
 USAGE_FLAG_DEVICE_ARGS = 0x01
@@ -35,23 +35,25 @@ USAGE_FLAG_SUPRESS_G_DEBUG_FLAG = 0x02
 USAGE_FLAG_FILE_ARGS = 0x04
 
 
-
+uiscan=False
 class Module(object):
     def __init__(self, mod, title, version, doc,
                  usage_data=None, avail_modes=None,
                  supported_ui_toolkits=None,
-                 run_as_root_ok=False, quiet=False):
+                 run_as_root_ok=False, quiet=False, deprecated=False):
 
         self.mod = mod
         self.title = title
         self.version = version
         self.doc = doc
         self.usage_data = usage_data
-        os.umask(0037)
+        os.umask(0o037)
         log.set_module(mod)
         self.args = []
         self.quiet = quiet
+        self.deprecated = deprecated
         self.lock_file = None
+        self.help_only_support = False
         prop.prog = sys.argv[0]
 
         if os.getenv("HPLIP_DEBUG"):
@@ -77,6 +79,10 @@ class Module(object):
             self.installed_ui_toolkits.append(UI_TOOLKIT_QT4)
             self.num_installed_ui_toolkits += 1
 
+        if utils.to_bool(sys_conf.get('configure', 'qt5', '0')):
+            self.installed_ui_toolkits.append(UI_TOOLKIT_QT5)
+            self.num_installed_ui_toolkits += 1
+
         self.default_mode = INTERACTIVE_MODE
 
         self.num_valid_modes = 0
@@ -107,8 +113,8 @@ class Module(object):
                 self.default_ui_toolkit = 'none'
 
             elif (UI_TOOLKIT_QT4 in self.supported_ui_toolkits and self.default_ui_toolkit == 'qt4' and UI_TOOLKIT_QT4 in self.installed_ui_toolkits) or \
-                 (UI_TOOLKIT_QT3 in self.supported_ui_toolkits and self.default_ui_toolkit == 'qt3' and UI_TOOLKIT_QT3 in self.installed_ui_toolkits):
-
+                 (UI_TOOLKIT_QT3 in self.supported_ui_toolkits and self.default_ui_toolkit == 'qt3' and UI_TOOLKIT_QT3 in self.installed_ui_toolkits) or \
+                 (UI_TOOLKIT_QT5 in self.supported_ui_toolkits and self.default_ui_toolkit == 'qt5' and UI_TOOLKIT_QT5 in self.installed_ui_toolkits):
                 self.default_mode = GUI_MODE
 
             elif self.default_ui_toolkit == 'qt3' and UI_TOOLKIT_QT3 not in self.supported_ui_toolkits:
@@ -116,6 +122,9 @@ class Module(object):
                 if UI_TOOLKIT_QT4 in self.supported_ui_toolkits and UI_TOOLKIT_QT4 in self.installed_ui_toolkits: # (e.g, hp-linefeedcal?)
                     self.default_ui_toolkit = 'qt4'
                     self.default_mode = GUI_MODE
+                if UI_TOOLKIT_QT5 in self.supported_ui_toolkits and UI_TOOLKIT_QT5 in self.installed_ui_toolkits:
+                    self.default_ui_toolkit = 'qt5'
+                    self.default_mode = GUI_MODE
 
                 elif INTERACTIVE_MODE in self.avail_modes:
                     self.default_mode = INTERACTIVE_MODE
@@ -125,7 +134,8 @@ class Module(object):
 
                 else:
                     log.error("%s cannot be run using Qt3 toolkit." % self.mod)
-                    sys.exit(1)
+#                    sys.exit(1)
+                    self.help_only_support = True
 
             elif self.default_ui_toolkit == 'qt4' and UI_TOOLKIT_QT4 not in self.supported_ui_toolkits:
 
@@ -141,7 +151,8 @@ class Module(object):
 
                 else:
                     log.error("%s cannot be run using Qt4 toolkit." % self.mod)
-                    sys.exit(1)
+#                    sys.exit(1)
+                    self.help_only_support = True
 
 
         self.mode = self.default_mode
@@ -201,12 +212,16 @@ class Module(object):
             if UI_TOOLKIT_QT4 in self.supported_ui_toolkits and UI_TOOLKIT_QT4 in self.installed_ui_toolkits:
                 content.append(utils.USAGE_USE_QT4)
 
+            if UI_TOOLKIT_QT5 in self.supported_ui_toolkits and UI_TOOLKIT_QT5 in self.installed_ui_toolkits:
+                content.append(utils.USAGE_USE_QT5)
+                
+
         content.append(utils.USAGE_LOGGING1)
         content.append(utils.USAGE_LOGGING2)
         if include_flags & USAGE_FLAG_SUPRESS_G_DEBUG_FLAG != USAGE_FLAG_SUPRESS_G_DEBUG_FLAG:
             content.append(utils.USAGE_LOGGING3) # Issue with --gg in hp-sendfax
 
-        # -q/--lang
+        # --loc/--lang
         #if self.avail_modes is not None and GUI_MODE in self.avail_modes and prop.gui_build:
         #    content.append(utils.USAGE_LANGUAGE)
 
@@ -296,6 +311,7 @@ class Module(object):
         show_usage = None
         device_uri = None
         printer_name = None
+        uiscan=False
         error_msg = []
         mode = self.default_mode
         if prop.gui_build:
@@ -306,7 +322,7 @@ class Module(object):
 
         try:
             opts, self.args = getopt.getopt(sys.argv[1:], params, long_params)
-        except getopt.GetoptError, e:
+        except getopt.GetoptError as e:
             error_msg = [e.msg]
 
         else:
@@ -353,6 +369,9 @@ class Module(object):
 
                 elif o == '--help-desc':
                     show_usage = 'desc'
+                
+                elif o == '--uiscan':
+                    uiscan = True
 
                 elif o in ('--qt3', '--use-qt3'):
                     if self.avail_modes is not None and GUI_MODE in self.avail_modes:
@@ -375,8 +394,20 @@ class Module(object):
                             ui_toolkit = 'qt4'
                         else:
                             error_msg.append("%s does not support Qt4. Unable to enter GUI mode." % self.mod)
+ 
+                elif o in ('--qt5', '--use-qt5'):
+                    if self.avail_modes is not None and GUI_MODE in self.avail_modes:
+                        if self.supported_ui_toolkits is not None and \
+                            UI_TOOLKIT_QT5 in self.supported_ui_toolkits and prop.gui_build and \
+                            UI_TOOLKIT_QT5 in self.installed_ui_toolkits:
 
-                #elif o in ('-q', '--lang', '--loc'):
+                            mode = GUI_MODE
+                            ui_toolkit = 'qt5'
+                        else:
+                            error_msg.append("%s does not support Qt4. Unable to enter GUI mode." % self.mod)
+               
+
+                #elif o in ('--lang', '--loc'):
                 #    if a.strip() == '?':
                 #        utils.log_title(self.title, self.version)
                 #        self.showLanguages()
@@ -387,7 +418,16 @@ class Module(object):
         if error_msg:
             show_usage = 'text'
 
-        self.usage(show_usage, error_msg)
+        if self.help_only_support:
+            if show_usage or error_msg:
+                if uiscan == False:
+                    self.usage(show_usage, error_msg)
+            else:
+                log.info(log.bold("\nPlease check usage '%s --help'"%self.mod))
+                show_usage = 'text'
+        else:
+            if uiscan == False:
+                self.usage(show_usage, error_msg)
 
         if show_usage is not None:
             sys.exit(0)
@@ -399,7 +439,7 @@ class Module(object):
     def showLanguages(self):
         f = tui.Formatter()
         f.header = ("Language Code", "Alternate Name(s)")
-        for loc, ll in supported_locales.items():
+        for loc, ll in list(supported_locales.items()):
             f.add((ll[0], ', '.join(ll[1:])))
 
         f.output()
@@ -416,7 +456,7 @@ class Module(object):
             log.info()
 
         if show_usage == 'desc':
-            print self.doc
+            print(self.doc)
 
         else:
             utils.format_text(self.usage_data, show_usage, self.title, self.mod, self.version)
@@ -438,17 +478,23 @@ class Module(object):
             log.info("")
 
             if show_ver:
-                log.info(log.bold("HP Linux Imaging and Printing System (ver. %s)" % prop.version))
+                #if uiscan == False:
+                 log.info(log.bold("HP Linux Imaging and Printing System (ver. %s)" % prop.version))
             else:
-                log.info(log.bold("HP Linux Imaging and Printing System"))
+                #if uiscan == False:
+                 log.info(log.bold("HP Linux Imaging and Printing System"))
 
+            #if uiscan == False:
             log.info(log.bold("%s ver. %s" % (self.title, self.version)))
             log.info("")
-            log.info("Copyright (c) 2001-13 Hewlett-Packard Development Company, LP")
+            log.info("Copyright (c) 2001-18 HP Development Company, LP")
             log.info("This software comes with ABSOLUTELY NO WARRANTY.")
             log.info("This is free software, and you are welcome to distribute it")
             log.info("under certain conditions. See COPYING file for more details.")
             log.info("")
+            if self.deprecated:
+                log.warn(log.bold("%s support is deprecated. Feature can be used as is. Fixes or updates will not be provided" %self.title))
+                log.info("")
 
 
     def getDeviceUri(self, device_uri=None, printer_name=None, back_end_filter=device.DEFAULT_BE_FILTER,
@@ -484,14 +530,17 @@ class Module(object):
         if devices is None:
             devices = device.getSupportedCUPSDevices(back_end_filter, filter)
             log.debug(devices)
+            if not devices and restrict_to_installed_devices:
+                log.error("No device found that support this feature.")
+                return None
 
         if device_uri is not None:
             if device_uri in devices:
                 device_uri_ok = True
 
             elif restrict_to_installed_devices:
-                log.error("Invalid device URI: %s" % device_uri)
-                device_uri = None
+                log.error("'%s' device doesn't support this feature (or) Invalid device URI" % device_uri)
+                return None
 
             else:
                 device_uri_ok = True
@@ -504,13 +553,18 @@ class Module(object):
                back_end, is_hp, bb, model, serial, dev_file, host, zc, port = \
                             device.parseDeviceURI(uri)
                log.debug("back_end=%s, is_hp=%s, bb=%s, model=%s, serial=%s, dev_file=%s, host=%s, zc=%s, port= %s" % (back_end, is_hp, bb, model, serial, dev_file, host, zc, port))
-               if printer_name.lower() == model.lower():
+               cups_printer = devices[uri]
+               if printer_name.lower() in [m.lower() for m in cups_printer]:
                    printer_name_ok = True 
                    printer_name_device_uri = device_uri = uri
                    device_uri_ok = True
             if printer_name_ok is not True: 
-               log.error("Invalid printer name: %s" % printer_name)
+               log.error("'%s' device doesn't support this feature (or) Invalid printer name" % printer_name)
                printer_name = None
+               if restrict_to_installed_devices:
+                    return None
+
+
 
         if device_uri is not None and printer_name is None and device_uri_ok: # Only device_uri specified
             device_uri_ret = device_uri
@@ -527,11 +581,13 @@ class Module(object):
             device_uri_ret = device.getDeviceURIByPrinterName(printer_name, scan_uri_flag)
 
         elif len(devices) == 1: # Nothing specified, and only 1 device avail.
-            device_uri_ret = devices.keys()[0]
-            log.info("Using device: %s\n" % device_uri_ret)
+            device_uri_ret = list(devices.keys())[0]
 
-        if device_uri_ret is None and self.mode == INTERACTIVE_MODE and len(devices):
-            device_uri_ret = tui.device_table(devices, scan_uri_flag)
+        if device_uri_ret is None and len(devices):
+            if self.mode == INTERACTIVE_MODE:
+                device_uri_ret = tui.device_table(devices, scan_uri_flag)
+            else:
+                device_uri_ret = list(devices.keys())[0]
 
         if device_uri_ret is not None:
             user_conf.set('last_used', 'device_uri', device_uri_ret)
@@ -540,14 +596,14 @@ class Module(object):
             if self.mode in (INTERACTIVE_MODE, NON_INTERACTIVE_MODE):
                 log.error("No device selected/specified or that supports this functionality.")
                 sys.exit(1)
-            else:
-                log.debug("No device selected/specified")
+#            else:
+#                log.debug("No device selected/specified")
 
         return device_uri_ret
 
 
     def getPrinterName(self, printer_name, device_uri, back_end_filter=device.DEFAULT_BE_FILTER,
-                       filter=device.DEFAULT_FILTER):
+                       filter=device.DEFAULT_FILTER, restrict_to_installed_devices=True):
         """ Validate passed in parameters, and, if in text mode, have user select desired printer to use.
             Used for tools that are printer queue-centric and accept -p (and maybe also -d).
             Use the filter(s) to restrict what constitute valid printers.
@@ -575,14 +631,20 @@ class Module(object):
         printers = device.getSupportedCUPSPrinterNames(back_end_filter, filter)
         log.debug(printers)
 
+        if not printers:
+            log.error("No device found that support this feature.")
+            return False, None, None
+
         if device_uri is not None:
             devices = device.getSupportedCUPSDevices(back_end_filter, filter)
             if device_uri in devices:
                 device_uri_ok = True
                 device_uri_ret = device_uri
             else:
-                log.error("Invalid device URI: %s" % device_uri)
+                log.error("'%s' device doesn't support this feature (or) Invalid device URI" % device_uri)
                 device_uri = None
+                if restrict_to_installed_devices:
+                    return False, None, None
 
         if printer_name is not None:
             if printer_name == '*':
@@ -595,12 +657,14 @@ class Module(object):
                     printer_name = None
 
             else:
-                if printer_name in printers:
+                if printer_name.lower() in [p.lower() for p in printers]:
                     printer_name_ok = True
                     device_uri_ret = device.getDeviceURIByPrinterName(printer_name)
                 else:
-                    log.error("Invalid printer name")
+                    log.error("'%s' device doesn't support this feature (or) Invalid printer name" % printer_name)
                     printer_name = None
+                    if restrict_to_installed_devices:
+                        return False, None, None
 
         if device_uri is not None and printer_name is None and device_uri_ok: # Only device_uri specified
             if len(devices[device_uri]) == 1:
@@ -618,7 +682,6 @@ class Module(object):
 
         elif len(printers) == 1: # nothing specified, and only 1 avail. printer
             printer_name_ret = printers[0]
-            log.info("Using printer: %s\n" % printer_name_ret)
 
         if printer_name_ret is None and self.mode in (INTERACTIVE_MODE, NON_INTERACTIVE_MODE) and len(printers):
             printer_name_ret = tui.printer_table(printers)
@@ -639,7 +702,7 @@ class Module(object):
             else:
                 log.debug("No printer selected/specified")
 
-        return printer_name_ret, device_uri_ret
+        return True, printer_name_ret, device_uri_ret
 
 
     def lockInstance(self, suffix='',suppress_error=False):
