@@ -589,7 +589,7 @@ class SetupDialog(QDialog, Ui_Dialog):
             if self.bus == 'net' and utils.check_lan():
                 FailureUI(self, self.__tr('''<b>HPLIP cannot detect printers in your network.</b><p>This may be due to existing firewall settings blocking the required ports.
                 When you are in a trusted network environment, you may open the ports for network services like mdns and slp in the firewall. For detailed steps follow the link.
-                <b>http://hplipopensource.com/node/374</b></p>'''),
+                <b>https://developers.hp.com/hp-linux-imaging-and-printing/KnowledgeBase/Troubleshooting/TroubleshootNetwork</b></p>'''),
                         self.__tr("HP Device Manager"))
             
 
@@ -1011,11 +1011,11 @@ class SetupDialog(QDialog, Ui_Dialog):
 
     def setAddPrinterButton(self):
         '''
-        If the device is on usb we assign new ipp uri to the printer. 
+        If the device is on usb and os doesn't supports ipp-usb we assign new ipp uri to the printer. 
         this is for driverless usb printer held by ippusbxd service on ubuntu 20 and above.
         For network devices follow the old code.
         '''
-        if(self.bus == "usb"):
+        if(self.bus == "usb" and not os.path.isdir('/usr/share/ipp-usb/quirks')):
             from base import local
             from base.local import detectLocalDevices
             try:
